#include "common.h"
#include "audio.h"
#include "nvd.h" 
#include "sensor.h" 

u8 gnAudioMode = AUDIO_MODE_SPEED;
s16 gnKLpf = 0;
s16 gnAccel = 0;

s16 gnBeepPeriodTicks = 0;
s16 gnBeepTransition;
s32 gnBeepCps = 0;
s32 gnVarioCps = 0;

void audio_Config(void) {
    OC1CONbits.OCTSEL = 0; // use TMR2
	}

void audio_SetFreq(s32 nFreqHz) {
   u16 tmrTicks;
   OC1CONbits.OCM = 0; // disable ocm
   T2CONbits.TON = 0; // stop tmr2
   TMR2 = 0;
   if (nFreqHz) {
      tmrTicks = (u16)((FOSC_HZ>>3)/nFreqHz);
      OC1R = (tmrTicks >> 1); // square wave
      OC1RS = tmrTicks;
      PR2 = tmrTicks;
      }
    else { // dc 0V
      OC1R = 50;
      OC1RS = 100;
      PR2 = 40;
      }
   T2CONbits.TON = 1; // start tmr2
   OC1CONbits.OCM = 5; // continuous pulse mode
   }



    
//  +---+---+---+ BeepPeriod +----+----+----+----+----+----+----+
//                                +----+----+--BeepTransition---+
//
//  Each tick is one pressure value sampling cycle  (approximately 1/15 seconds).
//  A counter is initialized to the value BeepPeriod at the start of the tone generation. 
//  Each tick, the counter is decremented. When it reaches the value BeepTransition, the tone
//  is shut off. The whole cycle repeats when the counter reaches zero.

// use this table for changing beep repeat rate based on vertical acceleration/deceleration
const s16 gnModeABeepTbl[5][2] = {
{6,1}, 
{5,1},
{4,1}, 
{3,1}, 
{2,1} 
};

void sns_LPFAccel(void)   {
   	s16 tmp;
	static s16 nAccelx32 = 0;
   	tmp = nAccelx32*KLPF_ACCEL + (gnAccel<<5)*(32-KLPF_ACCEL);
   	nAccelx32 = (tmp >= 0 ? ((tmp + 16)>>5) : ((tmp - 16)>>5));
   	gnAccel = (nAccelx32>>5);
   	}  

void sns_LPFClimbRate(void)   {
   	s16 tmp,cps;
	static s16 nCpsx32 = 0;
	cps = (s16)gnCps;
   	tmp = nCpsx32*gnKLpf + (cps<<5)*(32-gnKLpf);
   	nCpsx32 = (tmp >= 0 ? ((tmp + 16)>>5) : ((tmp - 16)>>5));
   	gnCps = (s32)(nCpsx32>>5);
   	}  

// use this table for changing beep repeat rate based on vertical speed
const s16 gnModeSBeepTbl[20][2] = {
{6,1}, 
{5,1},
{4,1}, 
{4,1}, 
{3,1}, 
{3,1}, 
{3,1}, 
{3,1}, 
{2,1}, 
{2,1}, 
{2,1},
{2,1}, 
{2,1}, 
{2,1}, 
{2,1}, 
{2,1}, 
{2,1}, 
{2,1}, 
{2,1},
{2,1}  
};


void audio_Beep(s32 nCps) {
   static s32 nFreqHz;
   static s32 nOldCps = 0;
   int nRange;

	gnAccel = (s16)(nCps - nOldCps);
    sns_LPFAccel();
    nOldCps = nCps;

   if ((gnBeepPeriodTicks <= 0) || (ABS(nCps-gnVarioCps) > BEEP_DISCR_THRESHOLD) || ((nCps > 0) && (gnVarioCps <= 0)) ) {
      gnVarioCps = nCps;
      // if sinking much faster than glider sink rate, generate continuous tone alarm
      if (gnVarioCps <= gnSinkToneCps) {
         gnBeepCps = gnVarioCps;	
         nFreqHz = VARIO_SINK_FREQHZ;
         audio_SetFreq(nFreqHz);
         gnBeepPeriodTicks = 0;
         gnBeepTransition  = 0;
         }
      //if climbing, generate beep
      else {
         gnBeepCps = gnVarioCps - gnClimbToneCps;
         if (gnBeepCps > 0) {
            if (gnBeepCps > VARIO_MAX_CPS) {
				gnBeepCps = VARIO_MAX_CPS;
				}
		 	if (gnAudioMode == AUDIO_MODE_SPEED_ACCEL) {
            	nRange = ABS(gnAccel)/7;
            	if (nRange > 4) nRange = 4;
            	gnBeepPeriodTicks = gnModeABeepTbl[nRange][0];
            	gnBeepTransition = gnModeABeepTbl[nRange][1];
				}
			else { // gnAudioMode == AUDIO_MODE_SPEED
            	nRange = gnBeepCps/50;
            	if (nRange > 19) nRange = 19;
            	gnBeepPeriodTicks = gnModeSBeepTbl[nRange][0];
            	gnBeepTransition = gnModeSBeepTbl[nRange][1];
				}	
         	if (gnBeepCps > VARIO_XOVER_CPS) {
               nFreqHz = VARIO_XOVER_FREQHZ + ((gnBeepCps - VARIO_XOVER_CPS)*(VARIO_MAX_FREQHZ - VARIO_XOVER_FREQHZ))/(VARIO_MAX_CPS - VARIO_XOVER_CPS);
               }
            else {
               nFreqHz = VARIO_MIN_FREQHZ + (gnBeepCps*(VARIO_XOVER_FREQHZ - VARIO_MIN_FREQHZ))/VARIO_XOVER_CPS;
               }
            audio_SetFreq(nFreqHz);
            }
      // if  within normal glider sink rate, be quiet
         else{
            gnBeepPeriodTicks = 0;
            gnBeepTransition  = 0;
            nFreqHz = 0;
            audio_SetFreq(nFreqHz);
            }
         }
      }
   else{ 
      gnBeepPeriodTicks--;
      if (gnBeepPeriodTicks <= gnBeepTransition){ // shut off tone
         nFreqHz = 0;
         audio_SetFreq(nFreqHz);
         }
#ifdef AUDIO_CHIRP
	  else {
		nFreqHz += 10;
        audio_SetFreq(nFreqHz);
		}
#endif
	  }
   }

